%% Program description
% This program generates a matrix of expected casflows for a given CDO
% contract using Monte Carlo simulation. It uses the function CDO_cashflow_generator 
% for all possible default rates.
% The element (i,j) of the matrix represents the expected cashflow of the
% tranche i with the default rate d_j.

%% Input Data

% Tranche attachment points
attachment_points = [0; 0.03; 0.06; 0.09; 0.12; 0.22; 1];

% Tranche prices (periodical) (1bp = 0.0001)
%tranche_prices_bid = [0.0500; 0.99*0.0160; 0.99*0.0080; 0.99*0.0058; 0.99*0.0040; 0.95*0.0010]; %hull 2008
%tranche_prices_ask = [0.0500; 1.01*0.0160; 1.01*0.0080; 1.01*0.0058; 1.01*0.0040; 1.05*0.0010]; %hull 2008

tranche_prices_bid = [0.0500; 0.005375; 0.0014; 0.000575; 0.000213; 0.00008]; %halperin 5yr
tranche_prices_ask = [0.0500; 0.005525; 0.00155; 0.000675; 0.000288; 0.00013]; %halperin 5yr

%tranche_prices_bid = [0.0500; 0.0130; 0.003675; 0.00165; 0.00055; 0.00024]; %halperin 7yr
%tranche_prices_ask = [0.0500; 0.0132; 0.00385; 0.0018; 0.00065; 0.00029]; %halperin 7yr

%tranche_prices_bid = [0.0500; 0.0348; 0.0093; 0.0040; 0.00135; 0.000435]; %halperin 10yr
%tranche_prices_ask = [0.0500; 0.0353; 0.0095; 0.0042; 0.001425; 0.000485]; %halperin 10yr

% Up front trache payments
%equity_tranche_up_front_payment_bid = 0.98*0.23; %hull 2008
%equity_tranche_up_front_payment_ask = 1.02*0.23; %hull 2008

equity_tranche_up_front_payment_bid = 0.1175; %halperin 5yr
equity_tranche_up_front_payment_ask = 0.1200; %haplerin 5yr

%equity_tranche_up_front_payment_bid = 0.2688; %halperin 7yr
%equity_tranche_up_front_payment_ask = 0.2713; %haplerin 7yr

%equity_tranche_up_front_payment_bid = 0.4188; %halperin 10yr
%equity_tranche_up_front_payment_ask = 0.4213; %haplerin 10yr

% Contract period (years)
contract_period = 5;
% Number of default rates
I = 1000;

% Minimum hazard rate
d_min = 1e-8;

% Maximum hazard rate
d_max = 100;


% interest rate
interest_rate=0.04;

%% Simulation
for i=1:I
  step = i
  hazard_rate = exp(log(d_min)+(i-1)*(log(d_max)-log(d_min))/I);
  matrix_A_bid(:,i) = CDO_cashflow_generator(attachment_points, tranche_prices_bid, equity_tranche_up_front_payment_bid,...
                                         contract_period, hazard_rate, interest_rate);
  matrix_A_ask(:,i) = CDO_cashflow_generator(attachment_points, tranche_prices_ask, equity_tranche_up_front_payment_ask,...
                                         contract_period, hazard_rate, interest_rate);                                   
end


clear contract_period attachment_points tranche_prices_bid tranche_prices_ask equity_tranche_up_front_payment_bid equity_tranche_up_front_payment_ask I i d_max d_min hazard_rate interest_rate; 
clear step;